@@ -146,10 +146,8 @@ testpats = [
             r'\[[^\]]+==',
             '[ foo == bar ] is a bashism, use [ foo = bar ] instead',
         ),
-        (
-            r'(^|\|\s*)grep (-\w\s+)*[^|]*[(|]\w',
-            "use egrep for extended grep syntax",
-        ),
+        (r'(^|\|\s*)egrep', "use grep -E for extended grep syntax"),
+        (r'(^|\|\s*)fgrep', "use grep -F for fixed string grepping"),
         (r'(^|\|\s*)e?grep .*\\S', "don't use \\S in regular expression"),
         (r'(?<!!)/bin/', "don't use explicit paths for tools"),
         (r'#!.*/bash', "don't use bash in shebang, use sh"),
@@ -385,12 +383,6 @@ commonpypats = [
             "use True/False for constant Boolean expression",
         ),
         (r'^\s*if False(:| +and)', 'Remove code instead of using `if False`'),
-        (
-            r'(?:(?<!def)\s+|\()hasattr\(',
-            'hasattr(foo, bar) is broken on py2, use util.safehasattr(foo, bar) '
-            'instead',
-            r'#.*hasattr-py3-only',
-        ),
         (r'opener\([^)]*\).read\(', "use opener.read() instead"),
         (r'opener\([^)]*\).write\(', "use opener.write() instead"),
         (r'(?i)descend[e]nt', "the proper spelling is descendAnt"),
