@@ -11,10 +11,7 @@ import sys
 # to work when run from a virtualenv.  The modules were chosen empirically
 # so that the return value matches the return value without virtualenv.
 if True:  # disable lexical sorting checks
-    try:
-        import BaseHTTPServer as basehttpserver
-    except ImportError:
-        basehttpserver = None
+    import argparse
     import zlib
 
 import testparseutil
@@ -198,9 +195,8 @@ def populateextmods(localmods):
 def list_stdlib_modules():
     """List the modules present in the stdlib.
 
-    >>> py3 = sys.version_info[0] >= 3
     >>> mods = set(list_stdlib_modules())
-    >>> 'BaseHTTPServer' in mods or py3
+    >>> 'http' in mods
     True
 
     os.path isn't really a module, so it's missing:
@@ -217,7 +213,7 @@ def list_stdlib_modules():
     >>> 'collections' in mods
     True
 
-    >>> 'cStringIO' in mods or py3
+    >>> 'array' in mods
     True
 
     >>> 'cffi' in mods
@@ -244,7 +240,7 @@ def list_stdlib_modules():
     stdlib_prefixes = {sys.prefix, sys.exec_prefix}
     # We need to supplement the list of prefixes for the search to work
     # when run from within a virtualenv.
-    for mod in (basehttpserver, zlib):
+    for mod in (argparse, zlib):
         if mod is None:
             continue
         try:
