@@ -4,7 +4,6 @@
 # This software may be modified and distributed under the terms
 # of the BSD license. See the LICENSE file for details.
 
-from __future__ import absolute_import
 
 import cffi
 import distutils.ccompiler
@@ -53,7 +52,10 @@ SOURCES = [
 # Headers whose preprocessed output will be fed into cdef().
 HEADERS = [
     os.path.join(HERE, "zstd", *p)
-    for p in (("zstd.h",), ("dictBuilder", "zdict.h"),)
+    for p in (
+        ("zstd.h",),
+        ("dictBuilder", "zdict.h"),
+    )
 ]
 
 INCLUDE_DIRS = [
@@ -80,12 +82,20 @@ if hasattr(compiler, "initialize"):
 if compiler.compiler_type == "unix":
     args = list(compiler.executables["compiler"])
     args.extend(
-        ["-E", "-DZSTD_STATIC_LINKING_ONLY", "-DZDICT_STATIC_LINKING_ONLY",]
+        [
+            "-E",
+            "-DZSTD_STATIC_LINKING_ONLY",
+            "-DZDICT_STATIC_LINKING_ONLY",
+        ]
     )
 elif compiler.compiler_type == "msvc":
     args = [compiler.cc]
     args.extend(
-        ["/EP", "/DZSTD_STATIC_LINKING_ONLY", "/DZDICT_STATIC_LINKING_ONLY",]
+        [
+            "/EP",
+            "/DZSTD_STATIC_LINKING_ONLY",
+            "/DZDICT_STATIC_LINKING_ONLY",
+        ]
     )
 else:
     raise Exception("unsupported compiler type: %s" % compiler.compiler_type)
