@@ -817,8 +817,8 @@ def _dotransplant(ui, repo, *revs, **opts):
 
     sourcerepo = opts.get(b'source')
     if sourcerepo:
-        u = urlutil.get_unique_pull_path(b'transplant', repo, ui, sourcerepo)[0]
-        peer = hg.peer(repo, opts, u)
+        path = urlutil.get_unique_pull_path_obj(b'transplant', ui, sourcerepo)
+        peer = hg.peer(repo, opts, path)
         heads = pycompat.maplist(peer.lookup, opts.get(b'branch', ()))
         target = set(heads)
         for r in revs:
