@@ -31,6 +31,11 @@ for line in sys.stdin:
         elif not line.startswith(' '):
             state = 'none'
 
+        elif not line.replace('^', '').replace('~', '').strip():
+            # PEP 657: Fine-grained error locations in tracebacks
+            #                       ~~~~~~^^^^^^^^^
+            continue
+
     elif state == 'file':
         # Ignore lines after "  File "
         state = 'tb'
